/* ######################################################################### */
/* #                                                                       # */
/* #                                                                       # */
/* #                Copyright (c) 2005 Iskratel Ltd.                       # */
/* #                                                                       # */
/* #                                                                       # */
/* # File Name    : DefaultCodec.java                                      # */
/* #                                                                       # */
/* # File Code    :                                                        # */
/* #                                                                       # */
/* # Description  : Implementation of Default Codec class.                 # */
/* #                                                                       # */
/* # Revision     : 1.0                                                    # */
/* #                                                                       # */
/* # Project Name :                                                        # */
/* #                                                                       # */
/* # Date         : Oct 2005                                               # */
/* #                                                                       # */
/* # Author       : Andrej Ciglic ITWEV6                                   # */
/* #                                                                       # */
/* # Translation  : javac                                                  # */
/* #                                                                       # */
/* # Remarks      : Implementation of Iskratel TTCN-3 TCI part.            # */
/* #                                                                       # */
/* ######################################################################### */

package com.iskratel.ttcn.tci.codec;

import com.testingtech.ttcn.tci.codec.base.AbstractBaseCodec;
import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import org.etsi.ttcn.tci.RecordValue;
import org.etsi.ttcn.tci.TciCDProvided;
import org.etsi.ttcn.tci.TciTypeClass;
import org.etsi.ttcn.tci.Type;
import org.etsi.ttcn.tci.Value;
import org.etsi.ttcn.tri.TriMessage;





/**<!--#####################################################################-->
 * 
 * Class <b>DefaultCodec</b> is extending <b>AbstractBaseCodec</b> class
 * and provides Default Codec and Decodec.
 * <p>
 * @author Andrej Ciglic ITWEV6
 * 
 * <!--##################################################################--> */  
public class DefaultCodec extends AbstractBaseCodec implements TciCDProvided {

  private byte[] encodedMsg = null ;


  
  /**<!--###################################################################-->
   *
   * Constructor method <b>DefaultCodec</b> creates new object and prepares RB.
   * <p>
   * @param rb needed to resolve specific information (e.g. which logging to 
   *           use)
   *         
   * <!--################################################################--> */  
  public DefaultCodec(RB rb) {
    super(rb);
  }


  
  /**<!--###################################################################-->
   *
   * Method <b>encode</b> codes a message according to the native encoding
   * rules.
   * <p>
   * @param template value should be encoded.
   * <p>
   * @return <code>TriMessage</code> represents the collected byteArray
   *         message.
   * 
   * <!--################################################################--> */
  public TriMessage encode(Value template) {
    // The template is given to the AbstractBaseCodec which handles the basic encoding
    TriMessage encodedMessage = super.encode(template);
    return encodedMessage;
  }
  
  

  /**<!--###################################################################-->
   *
   * Method <b>decode</b> decodes a received message according to the native 
   * decoding rules.
   * <p>
   * @param rcvdMessage message that shall be decoded.
   * @param decodingHypothesis the decoding hypothesis against which the
   *        message will be decoded.
   * <p>
   * @return the received message decoded according to the decoding hypothesis
   *         or <code>null</code> if message was not of type
   *         <code>decodingHypothesis</code>.
   * 
   * <!--################################################################--> */
  public Value decode(TriMessage rcvdMessage, Type decodingHypothesis) {
    // Get the received byte array 
    encodedMsg = rcvdMessage.getEncodedMessage();
    // Set the bitpos to 0 (this is needed to know the actual bit position from the byte array)
    bitpos = 0;
    // The the encoded message is null, nothing can be done, so return null 
    if(encodedMsg.length == 0) return null;
	
    // Check wich type class the decoding hypothesis is of
    switch (decodingHypothesis.getTypeClass()) {
      case TciTypeClass.RECORD:
//        System.out.println("Decode \""+decodingHypothesis.getName()+"\" .");
	// If it is a record and the name is commonPacket, then we know how to decode
        //if (decodingHypothesis.getName().equals("Response")) {
        // Create a new instance
        RecordValue NewHypotesis = (RecordValue) decodingHypothesis.newInstance();
        // Fill the NewHypotesis
        //return createNewHypotesis(NewHypotesis,encodedMsg);
        return NewHypotesis;
        //} else {
        //  return null;
        //}
      default:
        return super.decode(rcvdMessage,decodingHypothesis);
    }   
  }

}
/* ######################################################################### */
/* # End of file                                                           # */
/* ######################################################################### */
