/* ######################################################################### */
/* #                                                                       # */
/* #                                                                       # */
/* #                Copyright (c) 2016 Sintesio Org.                       # */
/* #                                                                       # */
/* #                                                                       # */
/* # File Name    : DiameterAdapter.java                                   # */
/* #                                                                       # */
/* # File Code    :                                                        # */
/* #                                                                       # */
/* # Description  : Implementation of DIAMETER parser class.               # */
/* #                                                                       # */
/* # Revision     : 1.0                                                    # */
/* #                                                                       # */
/* # Project Name :                                                        # */
/* #                                                                       # */
/* # Date         : Dec 2016                                               # */
/* #                                                                       # */
/* # Author       : Sintesio                                               # */
/* #                                                                       # */
/* # Translation  : javac                                                  # */
/* #                                                                       # */
/* #                                                                       # */
/* ######################################################################### */

package com.iskratel.ttcn.tri;

//import java.net.URI;
import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
//import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.EnumeratedValue;
import org.etsi.ttcn.tci.TciCDProvided;
import org.etsi.ttcn.tci.TciCDRequired;
import org.etsi.ttcn.tci.TciModuleId;
import org.etsi.ttcn.tci.TciModuleParameterId;
import org.etsi.ttcn.tci.TciParameterList;
//import org.etsi.ttcn.tci.TciModuleParameterId;
import org.etsi.ttcn.tci.TciTMProvided;
import org.etsi.ttcn.tri.TriAddress;
import org.etsi.ttcn.tri.TriCommunicationSA;
import org.etsi.ttcn.tri.TriComponentId;
import org.etsi.ttcn.tri.TriFunctionId;
import org.etsi.ttcn.tri.TriMessage;
import org.etsi.ttcn.tri.TriParameter;
import org.etsi.ttcn.tri.TriParameterList;
import org.etsi.ttcn.tri.TriPlatformPA;
import org.etsi.ttcn.tri.TriPortId;
import org.etsi.ttcn.tri.TriStatus;

import com.testingtech.ttcn.tci.TciEncoding;
import com.testingtech.ttcn.tci.TciModuleParameterIdImpl;
//import com.testingtech.ttcn.tci.TciParameterImpl;
import com.testingtech.ttcn.tci.TciParameterListImpl;
//import com.testingtech.ttcn.tci.TciModuleParameterIdImpl;
import com.testingtech.ttcn.tci.TestCaseParameterServer;
import com.testingtech.ttcn.tri.TestAdapter;
import com.testingtech.ttcn.tri.TriStatusImpl;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;

//import com.iskratel.diameter.parser.*;
import com.iskratel.ttcn.tci.codec.DIAMETERCodec;



public class DiameterAdapter extends TestAdapter implements TriCommunicationSA,	TriPlatformPA, TciEncoding, TestCaseParameterServer {

	private static final long serialVersionUID = 1L;

	static Logger log = Logger.getLogger(DiameterAdapter.class);

	private Hashtable<String, LocalAdapterBase> adapterCollection = new Hashtable<String, LocalAdapterBase>();
	
	private TciCDRequired typeServer;

	private org.etsi.ttcn.tci.Type charstringType;
//	private org.etsi.ttcn.tci.Type integerType;
//    private org.etsi.ttcn.tci.Type floatType;
//    private org.etsi.ttcn.tci.Type booleanType;


	private static TciTMProvided management;

	private String portNameForCoding = "";
	
	private int TransportProtocol;//0-tcp, 1-sctp

	public DiameterAdapter() {
		super();
		PropertyConfigurator.configure("log4j.properties");
		log.info("DIAMETER Adapter instance created.");

	}

	public TestAdapter setRB(RB rb) {
		super.setRB(rb);
		typeServer = RB.getTciCDRequired();
		charstringType = typeServer.getCharstring();
//		integerType = typeServer.getInteger();
//		floatType = typeServer.getFloat();
//        booleanType = typeServer.getBoolean();
		management = RB.getTciTMProvided();	
		
		
		TciCDProvided codec;
		codec=new DIAMETERCodec(RB);
		codecs.put("DIAMETERCodec", codec);
		
		try
		{
		    TciModuleParameterId parameterId = TciModuleParameterIdImpl.valueOf("PX_DIAM_TRANSPORT_PROTOCOL", null);
		    TransportProtocol = ((EnumeratedValue)(management.tciGetModulePar(parameterId))).getInt();
		    log.info("Tranport protocol= " + ((TransportProtocol ==0 )? "TCP" : "SCTP"));
		}
		catch (Exception ex){
			log.error("Missing PIXIT! Default transport protocol will be set to TCP.");//PX_DIAM_TRANSPORT_PROTOCOL
			TransportProtocol =0;
		}
		
		return this;
	}

	public  TriStatus triMap(TriPortId comPortId,	TriPortId tsiPortId) {
		
		log.info("comPortId=" + comPortId.getPortName() + "; tsiPortId=" + tsiPortId.getPortName());
		
		CsaDef.triMap(comPortId, tsiPortId);

		String portName = tsiPortId.getPortName();
		LocalAdapterBase baseAdapter = adapterCollection.get(portName);

		if (baseAdapter != null) {
			return new TriStatusImpl(TriStatus.TRI_OK);
		}

		//SIP
		if (tsiPortId.getPortName().equals("UE_Gm")) {
			LoopBackConnection adapter = new LoopBackConnection(log, Cte);
			baseAdapter = adapter;}
		else
		if (tsiPortId.getPortName().equals("UE_Gm2")) {
			LoopBackConnection adapter = new LoopBackConnection(log, Cte);
			baseAdapter = adapter;}
		else
		if (tsiPortId.getPortName().equals("CSCF_Mw")) {
			LoopBackConnection adapter = new LoopBackConnection(log, Cte);
			baseAdapter = adapter;}
		else
			
		//DIAMETER	
		if(tsiPortId.getPortName().equals("AS_Sh") || tsiPortId.getPortName().equals("ASorOSA_Sh")){
			if (TransportProtocol == 0 )
			    { DiameterConnection adapter = new DiameterConnection(log, Cte);
			      adapter.setAddresses(management, portName);
				  baseAdapter = adapter;}
			else 
			    { DiameterConnectionSCTP adapter =  new DiameterConnectionSCTP(log, Cte);
			     adapter.setAddresses(management, portName);
				 baseAdapter = adapter;}
		}
		else
		if(tsiPortId.getPortName().equals("AS_Dh") || tsiPortId.getPortName().equals("ASorOSA_Dh")){
			if (TransportProtocol == 0 )
			    { DiameterConnection adapter = new DiameterConnection(log, Cte);
			      adapter.setAddresses(management, portName);
				  baseAdapter = adapter;}
			else 
			    { DiameterConnectionSCTP adapter =  new DiameterConnectionSCTP(log, Cte);
			     adapter.setAddresses(management, portName);
				 baseAdapter = adapter;}
		}
		else
		if(tsiPortId.getPortName().equals("HSS_Sh")){
				if (TransportProtocol == 0 )
				    { DiameterConnection adapter = new DiameterConnection(log, Cte);
				      adapter.setAddresses(management, portName);
					  baseAdapter = adapter;}
				else 
				    { DiameterConnectionSCTP adapter =  new DiameterConnectionSCTP(log, Cte);
				     adapter.setAddresses(management, portName);
					 baseAdapter = adapter;}
		}
		else
		if(tsiPortId.getPortName().equals("SLF_Dh")){
			if (TransportProtocol == 0 )
			    { DiameterConnection adapter = new DiameterConnection(log, Cte);
			      adapter.setAddresses(management, portName);
				  baseAdapter = adapter;}
			else 
			    { DiameterConnectionSCTP adapter =  new DiameterConnectionSCTP(log, Cte);
			     adapter.setAddresses(management, portName);
				 baseAdapter = adapter;}
		}
		else {
			log.error("Port not supported (" + portName +")!\n");
			//return new TriStatusImpl(TriStatus.TRI_ERROR);
			return new TriStatusImpl("Port not supported (" + portName +")!");
		}
		adapterCollection.put(portName, baseAdapter);
		
		return baseAdapter.triMap(comPortId, tsiPortId);
	}

	public TriStatus triUnmap(TriPortId comPortId, TriPortId tsiPortId) {
		
		log.info("comPortId=" + comPortId.getPortName() + "; tsiPortId="+ tsiPortId.getPortName());
		
		CsaDef.triUnmap(comPortId, tsiPortId);
		
		String portName = tsiPortId.getPortName();
		LocalAdapterBase baseAdapter = adapterCollection.get(portName);
		if (baseAdapter == null) {
			return new TriStatusImpl(TriStatus.TRI_ERROR);
		}
		
		adapterCollection.remove(portName);
		return baseAdapter.triUnmap();
	}

	public TriStatus triSAReset() {
		
		//log.info("SAReset started");

		if (!adapterCollection.isEmpty()) {
			
			//Set ac = adapterCollection.keySet();
			
			Enumeration keys = adapterCollection.keys();
			
			/*String [] keys = (String[]) adapterCollection.keySet().toArray();
			 for (int i=0; i<keys.length; i++) {*/
			
			do {
				try{
				String key = (String) keys.nextElement();
				LocalAdapterBase baseAdapter = adapterCollection.get(key);
				
				baseAdapter.triUnmap();				
				//log.debug("remove key = " + key);
				//log.debug("AdapterCollection (" + adapterCollection.size()	+ ")!");
				adapterCollection.remove(key);
				//log.debug("AdapterCollection (" + adapterCollection.size()	+ ")!");
				//log.debug("If removed key exists = " + adapterCollection.containsKey(key));
				//}
				}catch(Exception e){}
			
			} while (keys.hasMoreElements());
			adapterCollection.clear();
		} 
		//else {
		//	log.error("AdapterCollection is not empty after SAReset ("+ adapterCollection.size() + ")!");
		//}
		
		return super.triSAReset();
	}

	public TriStatus triSend(TriComponentId componentId, TriPortId tsiPortId,	TriAddress triAddress, TriMessage triMessage) {
		
		//log.info("componentId=" + componentId.getComponentName() + "; tsiPortId="+ tsiPortId.getPortName());
		
		String portName = tsiPortId.getPortName();
		portNameForCoding = portName;
		LocalAdapterBase baseAdapter = adapterCollection.get(portName);
		
		if (baseAdapter == null) {
			return new TriStatusImpl(TriStatus.TRI_ERROR);
		}

		return baseAdapter.triSend(triAddress, triMessage);
    }

	public TriStatus triExternalFunction(TriFunctionId functionId,	TriParameterList parameterList, TriParameter returnValue) {
		
		return new TriStatusImpl(TriStatus.TRI_OK);
	}

	public TciCDProvided getCodec(String s) {
		
		//log.info("getCodec(" + s + ")");
		
		if (s == null  || s.equals("")) {
			
			if (portNameForCoding.equals("") || portNameForCoding == null) {
				s = "DIAMETERCodec";				
			} 
			
			else if (portNameForCoding.startsWith("UE_Gm")){
				s = "SIPCodec";
			}else if (portNameForCoding.startsWith("CSCF_")){
				s = "SIPCodec";
			}
			
			else if (portNameForCoding.startsWith("CTF")){
				s = "DIAMETERCodec";
			} else if (portNameForCoding.startsWith("CDF")){
				s = "DIAMETERCodec";
			} else if (portNameForCoding.startsWith("OCF")){
				s = "DIAMETERCodec";
			}
		}

		TciCDProvided codec = super.getCodec(s);
		if (codec != null) {		
			return codec;
		}
 		if (s.equals("DIAMETERCodec")) {
 			codec = new DIAMETERCodec(RB);
		 	codecs.put(s, codec);
		}
		return codec;		
	}

	public TciParameterList getTestCaseParameters(TciModuleId moduleId,	String testCaseId) {
		
    	log.info("moduleId=" + moduleId + "; testCaseId=" + testCaseId);
		TciParameterListImpl tciparameterlistimpl = new TciParameterListImpl();
		return tciparameterlistimpl;
	}
	
	public final CharstringValue createCharstringValue(String s) {
		
		CharstringValue out = (CharstringValue) charstringType.newInstance();
		out.setString(s);
		return out;
	}

}
