package com.iskratel.ttcn.tri;

//import java.io.BufferedInputStream;
import java.io.IOException;
//import java.io.OutputStream;
import java.io.PrintStream;

import java.net.InetAddress;
//import java.net.ServerSocket;
//import java.net.Socket;
import java.net.SocketAddress;
//import java.net.SocketTimeoutException;
//import java.net.InetAddress;
import java.net.InetSocketAddress;

import java.nio.ByteBuffer;
import com.sun.nio.sctp.AbstractNotificationHandler;
import com.sun.nio.sctp.AssociationChangeNotification;
import com.sun.nio.sctp.AssociationChangeNotification.AssocChangeEvent;
import com.sun.nio.sctp.HandlerResult;
//import com.sun.nio.sctp.IllegalReceiveException;
import com.sun.nio.sctp.Notification;
import com.sun.nio.sctp.MessageInfo;
import com.sun.nio.sctp.SctpChannel;
import com.sun.nio.sctp.SctpServerChannel;
import com.sun.nio.sctp.ShutdownNotification;
//import com.sun.nio.sctp.SctpSocketOption;
import com.sun.nio.sctp.SctpStandardSocketOptions;

import org.apache.log4j.Logger;

public abstract class SCTP extends Thread {
	
	static  final int SCTP_PPID_DIAMETER = 46;

	static final int MAX_RECEIVE_BUFFER_SIZE = 2048;//32678;

	private static final int SOCKET_RECEIVE_TIMEOUT = 200; // milliseconds
	protected static final int STOP_THREAD_TIMEOUT = SOCKET_RECEIVE_TIMEOUT * 2;

	protected Logger log = null;
	
	protected SctpChannel sctpChannel;
	
	protected volatile boolean runThread = false;

	public SCTP(Logger log) {
		super();
		this.log = log;
	}

	//public synchronized Boolean connect() {		
	public  boolean server() {
		SctpServerChannel sctpServerChannel;
		
		try {
			log.debug("Start server: "+getLocalAddress()+":"+getLocalPort());

			SocketAddress serverSocketAddress = new InetSocketAddress(InetAddress.getByName(getLocalAddress()),getLocalPort());
		    sctpServerChannel =  SctpServerChannel.open().bind(serverSocketAddress);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
				 
		try {
			
			sctpChannel = sctpServerChannel.accept();
	    	sctpServerChannel.close();
	    	sctpChannel.setOption(SctpStandardSocketOptions.SCTP_NODELAY,true);
	    	sctpChannel.setOption(SctpStandardSocketOptions.SO_RCVBUF,100);
	    	
			// Start thread receiving messages
			runThread = true;
			start();
			return true;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			//return false;
		}
		return false;
	}
	
	//public synchronized Boolean connect() {		
	public  boolean connect() {		
		try {
			//log.info("Connecting SCTP ...");
			if (runThread == true) {
				log.error("SCTP object has already been connected");
				return false;
			}
			// Connect this SCTP client to remote server
			String remoteAddress = getRemoteAddress();
			int remotePort = getRemotePort();
			String localAddress = getLocalAddress();
			int localPort = getLocalPort();
			log.debug("Connecting to remote server: " + remoteAddress + ":" + remotePort);
			log.debug("with using local address   : " + localAddress + ":" + localPort);
	        
	    	 sctpChannel = SctpChannel.open();
	    	 InetSocketAddress serverAddr = new InetSocketAddress(remoteAddress,  remotePort);
		     sctpChannel.bind(new InetSocketAddress( localAddress,localPort));
	    	 
	    	 sctpChannel.setOption(SctpStandardSocketOptions.SCTP_NODELAY,true);
	    	 sctpChannel.setOption(SctpStandardSocketOptions.SO_RCVBUF,100);
	    	 
	    	 sctpChannel.connect(serverAddr, 1 ,1);
	    		    	 
	    	 // Start thread receiving messages
			 runThread = true;
			 start();
			 //log.error("SCTP connect started");
			return true;

		} catch (Throwable ex) {
			ex.printStackTrace();
		}
		return false;
	}

	public boolean send(byte [] buffer) {
	    
		final ByteBuffer byteBuffer = ByteBuffer.allocate(2048);
        final MessageInfo messageInfo = MessageInfo.createOutgoing(null, 0).payloadProtocolID(SCTP_PPID_DIAMETER);
        
        byteBuffer.put(buffer);
        byteBuffer.flip();

        try {
           sctpChannel.send(byteBuffer, messageInfo);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
	}
	
//	//public synchronized Boolean send(byte[] buffer, int offset, int length) {
//	public boolean send(byte[] buffer, int offset, int length) {
//		try {
//			//log.error("Before sending");
//			if ((sendStream == null) || (offset > buffer.length - 1)
//					|| ((offset + length) > buffer.length)) {
//				return false;
//			}
//			sendStream.write(buffer, 0, buffer.length);
//			//log.error("Sent");
//			return true;
//
//		} catch (Throwable ex) {
//			log.error("Exception: ", ex);
//		}
//		return false;
//	}

	//public synchronized Boolean disconnect() {		
	public boolean disconnect() {
		//log.info("TCP disconnect");
		try {
			if (runThread == true) {
				// stop receive thread
				runThread = false;
				join(STOP_THREAD_TIMEOUT);
				
				sctpChannel.shutdown();
				sctpChannel.close();
				
				sctpChannel = null;
				return true;
			}
			return true;

		} catch (Throwable ex) {
			ex.printStackTrace();
		}
		return false;
	}
	
	public void run() {
		
		byte[] buffer = new byte[MAX_RECEIVE_BUFFER_SIZE];
		int bufferReceived = 0;
		//AssociationHandler assocHandler = new AssociationHandler();
		MessageInfo messageInfo = null;
		
		while (runThread == true) {
			// Receive additional data
			try {
				//log.debug("Trying to read from receiveStream...");
				byte [] actuallyReceived = new byte[MAX_RECEIVE_BUFFER_SIZE - bufferReceived];
				ByteBuffer tmpBuffer = ByteBuffer.wrap(actuallyReceived);
				tmpBuffer.clear();
				
				ReceiveNotificationHandler handler=new ReceiveNotificationHandler(sctpChannel);
				messageInfo  = sctpChannel.receive(tmpBuffer,  null, handler);
								
				if (messageInfo != null && messageInfo.bytes()>0){// && tmpBuffer.limit() > 0){ 
					//log.debug("Received " + messageInfo.bytes() + " SCTP bytes: " + HexDump.toString(actuallyReceived));
					System.arraycopy(actuallyReceived, 0, buffer, bufferReceived, messageInfo.bytes());
					bufferReceived += messageInfo.bytes();
				}
			} catch (Exception ex) {
				//log.debug("Read timed out"+ex.getMessage());
				continue;
			}
		
			// Read one or more messages from buffer before receiving more data from network
			int msgLength=0;
			do {
				msgLength = receiveBuffer(buffer, bufferReceived);
				if (msgLength < 0) {
					log.debug("Error found in received data, exiting receive thread");
					return;
				}
				System.arraycopy(buffer, msgLength, buffer, 0, bufferReceived - msgLength);
				bufferReceived -= msgLength;                                        
			} while (msgLength > 0);
			
		}
	}
	
	abstract protected String getRemoteAddress();

	abstract protected int getRemotePort();
	
	abstract protected String getLocalAddress();

	abstract protected int getLocalPort();

	abstract protected int receiveBuffer(byte [] buffer, int bufferSize);

}

final class AssociationHandler extends AbstractNotificationHandler {
	
	public HandlerResult handleNotification(AssociationChangeNotification not, PrintStream stream) {
		if (not.event().equals(AssocChangeEvent.COMM_UP)) {
			int outbound = not.association().maxOutboundStreams();
			int inbound = not.association().maxInboundStreams();
			stream.printf("New association setup with %d outbound streams"
					+ ", and %d inbound streams.\n", outbound, inbound);
		}
		return HandlerResult.CONTINUE;
	}

	public HandlerResult handleNotification(ShutdownNotification not,
			PrintStream stream) {
		stream.printf("The association has been shutdown.\n");
		return HandlerResult.RETURN;
	}
}

class ReceiveNotificationHandler extends AbstractNotificationHandler<Object>
{
    SctpChannel channel;
    boolean receivedCommUp;  // false

    public ReceiveNotificationHandler(SctpChannel channel) {
        this.channel = channel;
    }

    public boolean receivedCommUp() {
        return receivedCommUp;
    }

    @Override
    public HandlerResult handleNotification(Notification notification, Object attachment) {
        fail("Unknown notification type");
        return HandlerResult.CONTINUE;
    }

    @Override
    public HandlerResult handleNotification(AssociationChangeNotification notification, Object attachment) {
        AssocChangeEvent event = notification.event();
        debug("AssociationChangeNotification");
        debug("  Association: " + notification.association());
        debug("  Event: " + event);

        if (event.equals(AssocChangeEvent.COMM_UP))
            receivedCommUp = true;

        if (channel == null)
            return HandlerResult.RETURN;
        return HandlerResult.CONTINUE;
    }


    @Override
    public HandlerResult handleNotification(ShutdownNotification notification, Object attachment) {
        debug("ShutdownNotification");
        debug("  Association: " + notification.association());
        return HandlerResult.CONTINUE;
    }
    
    boolean debug = true;
    volatile int passed = 0, failed = 0;

    void pass() {passed++;}
    void fail() {failed++; Thread.dumpStack();}
    void fail(String msg) {System.err.println(msg); fail();}

    void unexpected(Throwable t) {failed++; t.printStackTrace();}
    void check(boolean cond) {if (cond) pass(); else fail();}
    void check(boolean cond, String failMessage) {if (cond) pass(); else fail(failMessage);}
    void debug(String message) {if(debug) {System.out.println(Thread.currentThread() + " " + message); } }
}
