package com.iskratel.ttcn.tri;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.net.InetAddress;

import org.apache.log4j.Logger;

public abstract class TCP extends Thread {

	static final int MAX_RECEIVE_BUFFER_SIZE = 32678;

	private static final int SOCKET_RECEIVE_TIMEOUT = 200; // milliseconds
	protected static final int STOP_THREAD_TIMEOUT = SOCKET_RECEIVE_TIMEOUT * 2;

	protected Logger log = null;

	protected ServerSocket serverSocket= null;
	protected Socket socket = null;
	
	protected OutputStream sendStream = null;
	protected BufferedInputStream receiveStream = null;

	protected volatile boolean runThread = false;

	public TCP(Logger log) {
		super();
		this.log = log;
	}

	//public synchronized Boolean connect() {		
	public  boolean server() {
		
		try {
			log.debug("Start server: "+getLocalAddress()+":"+getLocalPort());
			serverSocket = new ServerSocket(getLocalPort(),1,InetAddress.getByName(getLocalAddress()));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			log.error("Exception: ");
			return false;
		}
				 
		try {
			//log.error("Start accept!");
			socket = serverSocket.accept();
			String remoteip = socket.getInetAddress().getHostAddress();//  getRemoteSocketAddress().toString();
			//log.debug(remoteip + ":");
			log.debug(this.getRemoteAddress()+ ":");
			
			while ( !(remoteip.equals(this.getRemoteAddress()))){
				// close sockets
				socket.setSoLinger(false, 0);
				socket.shutdownInput();
				socket.shutdownOutput();
				socket.close();
				
				socket = serverSocket.accept();
				remoteip = socket.getInetAddress().getHostAddress();//  getRemoteSocketAddress().toString();
				//log.debug(remoteip + ":");
				//log.debug(this.getRemoteAddress()+ ":");
				
			    
			}
			
				serverSocket.close();
				socket.setSoTimeout(SOCKET_RECEIVE_TIMEOUT);
		        sendStream = socket.getOutputStream();
		    	receiveStream = new BufferedInputStream(socket.getInputStream());
				// Start thread receiving messages
				runThread = true;
				start();
				//return true;
			
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			log.error("Exception: ");
			return false;
		}
		return true;
	}
	
	//public synchronized Boolean connect() {		
	public  boolean connect() {		
		try {
			//log.info("Connecting TCP ...");
			if (runThread == true) {
				log.error("TCP object has already been connected");
				return false;
			}
			// Connect this TCP client to remote server
			String remoteAddress = getRemoteAddress();
			int remotePort = getRemotePort();
			String localAddress = getLocalAddress();
			int localPort = getLocalPort();
			log.debug("Connecting to remote server: " + remoteAddress + ":" + remotePort);
			log.debug("with using local address   : " + localAddress + ":" + localPort);
	        //socket = new Socket(remoteAddress, remotePort);
	        socket = new Socket(remoteAddress, remotePort,InetAddress.getByName(localAddress),localPort);
	        socket.setSoTimeout(SOCKET_RECEIVE_TIMEOUT);
	        sendStream = socket.getOutputStream();
	    	receiveStream = new BufferedInputStream(socket.getInputStream());
			// Start thread receiving messages
			runThread = true;
			start();
			//log.error("TCP connect started");
			return true;

		} catch (Throwable ex) {
			//ex.printStackTrace();
			log.error("Exception: ");
		}
		return false;
	}

	public boolean send(byte [] buffer) {
	    //log.info("Sending over TCP");
		return send(buffer, 0, buffer.length);
	}
	
	//public synchronized Boolean send(byte[] buffer, int offset, int length) {
	public boolean send(byte[] buffer, int offset, int length) {
		try {
			//log.error("Before sending");
			if ((sendStream == null) || (offset > buffer.length - 1)
					|| ((offset + length) > buffer.length)) {
				return false;
			}
			sendStream.write(buffer, 0, buffer.length);
			//log.error("Sent");
			return true;

		} catch (Throwable ex) {
			//log.error("Exception: ", ex);
			log.error("Exception: ");
		}
		return false;
	}

	//public synchronized Boolean disconnect() {		
	public boolean disconnect() {
		//log.info("TCP disconnect");
		try {
			if (runThread == true) {
				// stop receive thread
				runThread = false;
				join(STOP_THREAD_TIMEOUT);
				// close sockets
				socket.setSoLinger(false, 0);
				socket.shutdownInput();
				socket.shutdownOutput();
				socket.close();
				// release resources
				socket = null;
				receiveStream = null;
				sendStream = null;
				return true;
			}
			return true;

		} catch (Throwable ex) {
			//ex.printStackTrace();
			log.error("Exception: ");
		}
		return false;
	}
	
	public void run() {
		try {
			//log.error("TCP starts to run");
			if (receiveStream == null) {
				log.error("Connection has not been initialized");
				return;
			}
			byte[] buffer = new byte[MAX_RECEIVE_BUFFER_SIZE];
			int bufferReceived = 0;

			while (runThread == true) {
				// Receive additional data
				try {
					//log.debug("Trying to read from receiveStream...");
					byte[] tmpBuffer = new byte[MAX_RECEIVE_BUFFER_SIZE - bufferReceived];
					int received = receiveStream.read(tmpBuffer, 0,	MAX_RECEIVE_BUFFER_SIZE - bufferReceived);
					//log.debug("Read " + received + " bytes");
					if (received > 0) {
						byte [] actuallyReceived = new byte[received];
						System.arraycopy(tmpBuffer, 0, actuallyReceived, 0, received);
						//log.debug("Received " + received + " TCP bytes: " + HexDump.toString(actuallyReceived));
						System.arraycopy(actuallyReceived, 0, buffer, bufferReceived, received);
						bufferReceived += received;
					}
				} catch (SocketTimeoutException ex) {
					//log.debug("Read timed out");
					continue;
				}
				// Read one or more messages from buffer before receiving more
				// data from network
                 
                int msgLength=0;
				do {
					msgLength = receiveBuffer(buffer, bufferReceived);
					if (msgLength < 0) {
						log.debug("Error found in received data, exiting receive thread");
						return;
					}
					System.arraycopy(buffer, msgLength, buffer, 0, bufferReceived - msgLength);
					bufferReceived -= msgLength;                                        
				} while (msgLength > 0);
			}
			//log.error("TCP finished to run");
		} catch (Throwable ex) {
			log.error("Exception: ", ex);
			//log.error("Exception: ");
		}
	}

	abstract protected String getRemoteAddress();

	abstract protected int getRemotePort();
	
	abstract protected String getLocalAddress();

	abstract protected int getLocalPort();

	abstract protected int receiveBuffer(byte [] buffer, int bufferSize);

}
